//************************************************************
// OVERFILL SENSOR
// (C) Bernardo Giovanni
// http://www.settorezero.com
//
// Tutti i diritti riservati, vietata la
// ridistribuzione e/o riutilizzo anche parziale
// del codice senza consenso scritto dell'autore.
// Al seguente codice si applicano tutte le regole gi
// esposte nel disclaimer all'indirizzo:
// http://www.settorezero.com/wordpress/info/disclaimer/
//
//************************************************************
// Overfill sensor for polar liquids
// Sensore di troppo pieno per liquidi polari
// VERSION 2 - Edit of 20/08/2014
//************************************************************
//
// module:      main.c
// author:      Bernardo Giovanni
// mcu:         PIC12F675
// fclock:      internal @ 4MHz
// board:       custom
// compiler:    Microchip XC8 (1.31)
// ide:         MPLAB X (2.05)
// programmer:  PICkit 2
//
//************************************************************

#define _XTAL_FREQ 4000000
#include <xc.h>
#include "conf_12f675.h"

// I/O ports defines
#define BTN_RESET   GPIObits.GPIO3
#define LED_ALARM   GPIObits.GPIO5
#define BUZZER      GPIObits.GPIO4 // NOTE: active buzzer used!
#define PINM        GPIObits.GPIO1
#define SQWOUT      GPIObits.GPIO0
#define TIMEH       20

// global variables
bit alarm_flag=0;   // alarm active or not
unsigned char freqsqw=0; // square wave frequency counter
unsigned int valueprobe=0; // value from probe
unsigned int valueref=0; // value from trimmer
unsigned char measures=0; // number of measures
unsigned int buzzcount=0; // counter for buzzer intermittence
bit start_check=0;  // after first liquid contact, starts a counter
unsigned checkcount=0; // counter (confirm liquid contact after x seconds)
unsigned char ok=0;

// functions prototypes
void settings(void);
void __interrupt isr(void);

int main(void)
    {
    settings();

    while(1)
        {

        //if (!alarm_flag)
            //{
            if(PINM==1) // high level on measure pin (normally pulled down)
                {
                LED_ALARM=1;
                start_check=1; // starts counter
                while(PINM==1); // stay until pin is high
                if (checkcount>=(TIMEH-3))
                    {
                    ok++;
                    }
                else
                    {
                    ok=0;
                    
                    }
                start_check=0;
                checkcount=0;
                }
            LED_ALARM=0;
            //}

        if (ok>=150)
            {
            alarm_flag=1;
            ok=0;
            }

        // check reset button
        if (!BTN_RESET)
            {
            while(!BTN_RESET); // stay until button is released
            alarm_flag=0;
            buzzcount=0;
            start_check=0;
            checkcount=0;
            LED_ALARM=0;
            BUZZER=0;
            }

        } // end while(1)
    } // end main

void settings(void)
    {
    // digital functions
    // GP5 -> (out) led - level reached
    // GP4 -> (out) buzzer
    // GP3 -> (in) reset pushbutton (mclr as I/O can be used only as input!)
    // GP2 -> (in) A/D AN2 - level threshold
    // GP1 -> (in) A/D AN1 - probe input
    // GP0 -> (out) square wave out
    TRISIO=0x0E;
    GPIO5=0;
    GPIO4=0;
    GPIO0=0;

    // analog functions
    CMCONbits.CM=0b111; // comparator off
    ADCON0bits.ADFM = 1; // A/D result right-justified
    ADCON0bits.VCFG = 0; // Vdd as voltage reference
    ADCON0bits.CHS=0b10; // AN2 read first
    ADCON0bits.ADON = 1; // A/D module on
    ANSELbits.ANS=0b0100; // Analog/Digital select: GP3-AN2-GP1-GP0
    ANSELbits.ADCS=0b111; // A/D clock derived from internal oscillator

    // options
    OPTION_REGbits.nGPPU=1; // internal pull-ups resistors disabled
    OPTION_REGbits.T0CS=0; // clock source for TMR0 from internal clock (Fosc/4)
    OPTION_REGbits.PSA=0; // prescaler assigned to TMR0
    OPTION_REGbits.PS=0b001; // prescaler 1:4

        // interrupt configuration
    INTCONbits.TMR0IE=1; // interrupt on TMR0 overflow
    //INTCONbits.PEIE=0; // peripheral interrupt
    //PIE1bits.ADIE=0; // interrupt on A/D conversion end
    TMR0=6; // load timer0 register (interrupt every 1ms with PS1:4 and Fosc 4MHz)
    GIE=1; // enable global interrupts
    }

// interrupt vector
void __interrupt isr(void)
    {
    if (INTCONbits.T0IF) // interrupt on TMR0, every 1mS
        {
        T0IF=0; // interrupt reset
        TMR0=6; // timer0 reload

        // square wave out
        freqsqw++;
        if (freqsqw==TIMEH) //10mS ON, 10mS OFF => period=20mS Freq=50Hz
            {
            SQWOUT ^= 1;
            freqsqw=0;
            }

        // increment counter for re-check if re-check flag active
        if (start_check)
            {
            checkcount++;
            }

        // check alarm flag for turning on the buzzer
        if (alarm_flag)
            {
            buzzcount++;
            // Buzzer 500mS on, and 500mS OFF : BEEP BEEP BEEP[...]
            if (buzzcount<500)
                {
                BUZZER=1;
                }
            else
                {
                BUZZER=0;
                }
            if (buzzcount>2000)
                {
                buzzcount=0;
                }
            } // end alarm flag
        } // end tmr0 interrupt
    } // end isr


